package com.jiuqi.util;

import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Calendar;
import java.util.List;
import java.io.IOException;
import java.io.Reader;
import java.io.BufferedReader;
import java.io.StringReader;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.io.PrintStream;

public final class JqLib
{
    public static final String EMPTY_STRING = "";
    public static final String SQL_FALSE = "0";
    public static final String SQL_TRUE = "1";
    public static final char[] HEX_CHARS;
    public static final char[] BASE64_CHARS;
    private static final int[] DAYS_IN_MONTH;
    public static PrintStream testLogger;
    
    static {
        HEX_CHARS = new char[] { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F' };
        BASE64_CHARS = new char[] { 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '+', '/', '=' };
        DAYS_IN_MONTH = new int[] { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };
        JqLib.testLogger = null;
    }
    
    public static boolean isEmpty(final String value) {
        return value == null || value.equals("");
    }
    
    public static boolean isBoolTrue(final String sqlValue) {
        return sqlValue != null && (sqlValue.equals("1") || sqlValue.equalsIgnoreCase("T") || sqlValue.equalsIgnoreCase("TRUE"));
    }
    
    public static boolean isSymbol(final String s) {
        if (s == null || s.length() == 0) {
            return false;
        }
        for (int i = 0; i < s.length(); ++i) {
            if (!Character.isLetterOrDigit(s.charAt(i)) && '_' != s.charAt(i)) {
                return false;
            }
        }
        return true;
    }
    
    public static boolean isIdentifier(final String s) {
        if (s == null || s.length() == 0) {
            return false;
        }
        for (int i = 0; i < s.length(); ++i) {
            if (!Character.isLetterOrDigit(s.charAt(i)) && '_' != s.charAt(i)) {
                return false;
            }
        }
        return Character.isLetter(s.charAt(0)) || '_' == s.charAt(0);
    }
    
    public static boolean isNumberString(final String s) {
        if (s == null || s.length() == 0) {
            return false;
        }
        for (int i = 0; i < s.length(); ++i) {
            if (!Character.isDigit(s.charAt(i))) {
                return false;
            }
        }
        return true;
    }
    
    public static boolean hasSpaceChar(final String s) {
        for (int i = 0; i < s.length(); ++i) {
            if (Character.isSpaceChar(s.charAt(i))) {
                return true;
            }
        }
        return false;
    }
    
    public static String convInput(final String input) {
        try {
            return new String(input.getBytes("ISO8859_1"));
        }
        catch (final Exception ex) {
            return input;
        }
    }
    
    public static String quoteString(final String s) {
        return String.valueOf('\"') + s + '\"';
    }
    
    public static String quoteStringSingle(final String s) {
        return "'" + s + "'";
    }
    
    public static String toFixedString(final long iv, final int width) {
        final NumberFormat fmt = NumberFormatManager.createIntegerFormat(width);
        return fmt.format(iv);
    }
    
    public static String toFixedString(final long iv, final boolean seperator) {
        final NumberFormat fmt = NumberFormatManager.createNumberFormat(0, seperator);
        return fmt.format(iv);
    }
    
    public static String replaceString(final String s, final String sub, String rep) {
        if (rep == null) {
            rep = "";
        }
        final StringBuffer sb = new StringBuffer();
        int fromIndex = 0;
        int pos = 0;
        do {
            pos = s.indexOf(sub, fromIndex);
            if (pos >= 0) {
                if (pos > fromIndex) {
                    sb.append(s.substring(fromIndex, pos));
                }
                sb.append(rep);
                fromIndex = pos + sub.length();
            }
        } while (pos >= 0);
        if (fromIndex < s.length()) {
            sb.append(s.substring(fromIndex, s.length()));
        }
        return sb.toString();
    }
    
    public static String toFixedString(final double value, final boolean seperator) {
        final NumberFormat fmt = new DecimalFormat();
        fmt.setGroupingUsed(seperator);
        return fmt.format(value);
    }
    
    public static String strExclude(final String str, final char ch) {
        if (str == null) {
            return null;
        }
        final char[] chars = new char[str.length()];
        int index = 0;
        for (int i = 0; i < str.length(); ++i) {
            final char c = str.charAt(i);
            if (c != ch) {
                chars[index++] = c;
            }
        }
        return new String(chars, 0, index);
    }
    
    public static boolean charIsInCharSet(final char value, final char[] chrSet) {
        for (int i = 0; i < chrSet.length; ++i) {
            if (value == chrSet[i]) {
                return true;
            }
        }
        return false;
    }
    
    public static String strExclude(final String str, final char[] chs) {
        if (str == null) {
            return null;
        }
        if (chs == null) {
            return str;
        }
        final char[] chars = new char[str.length()];
        int index = 0;
        for (int i = 0; i < str.length(); ++i) {
            final char c = str.charAt(i);
            if (!charIsInCharSet(c, chs)) {
                chars[index++] = c;
            }
        }
        return new String(chars, 0, index);
    }
    
    public static String trimEx(final String src) {
        final StringBuffer ret = new StringBuffer(src);
        int i = ret.length() - 1;
        final int size = src.length();
        if (size == 0) {
            return "";
        }
        while (i >= 0) {
            if (ret.charAt(i) == '\n' || ret.charAt(i) == '\r') {
                ret.deleteCharAt(i);
            }
            --i;
        }
        return trim(ret.toString());
    }
    
    public static String trim(final String str) {
        if (str == null) {
            return null;
        }
        int s1 = -1;
        for (int i = 0; i < str.length() && (str.charAt(i) == ' ' || str.charAt(i) == '\u3000'); ++i) {
            s1 = i;
        }
        ++s1;
        int s2 = str.length();
        for (int j = s2 - 1; j >= s1 && (str.charAt(j) == ' ' || str.charAt(j) == '\u3000'); --j) {
            s2 = j;
        }
        if (s1 == 0 && s2 == str.length()) {
            return str;
        }
        if (s1 >= s2) {
            return "";
        }
        return str.substring(s1, s2);
    }
    
    public static String trimLeft(final String str) {
        if (str == null) {
            return null;
        }
        int s1 = 0;
        for (int i = 0; i < str.length() && str.charAt(i) == ' '; ++i) {
            s1 = i;
        }
        final int s2 = str.length();
        if (s1 == 0) {
            return str;
        }
        if (s1 >= s2) {
            return "";
        }
        return str.substring(s1, str.length());
    }
    
    public static String trimRight(final String str) {
        if (str == null) {
            return null;
        }
        int s2 = str.length();
        for (int i = s2 - 1; i >= 0 && str.charAt(i) == ' '; --i) {
            s2 = i;
        }
        if (s2 == str.length()) {
            return str;
        }
        if (s2 <= 0) {
            return "";
        }
        return str.substring(0, s2);
    }
    
    public static int compareString(final String s1, final String s2) {
        if (s1 == s2) {
            return 0;
        }
        if (s1 == null) {
            return -1;
        }
        if (s2 == null) {
            return 1;
        }
        return s1.compareTo(s2);
    }
    
    public static int compareArray(final int[] a1, final int[] a2) {
        if (a1 == a2) {
            return 0;
        }
        if (a1 == null) {
            return -1;
        }
        if (a2 == null) {
            return 1;
        }
        int result = a1.length - a2.length;
        if (result != 0) {
            return result;
        }
        for (int i = 0; i < a1.length; ++i) {
            result = a1[i] - a2[i];
            if (result != 0) {
                return result;
            }
        }
        return result;
    }
    
    public static String getLine(final String text, int linenum) {
        if (text == null) {
            return "";
        }
        final BufferedReader buffer = new BufferedReader(new StringReader(text));
        try {
            String line = null;
            while (linenum-- >= 0) {
                line = buffer.readLine();
            }
            return line;
        }
        catch (final IOException ex) {
            return text;
        }
    }
    
    public static String encodePassword(final String password, final int maxlength) {
        int s1 = 97;
        int s2 = 76;
        int len = (password == null) ? 0 : password.length();
        if (len > maxlength) {
            len = maxlength;
        }
        final StringBuffer pw = new StringBuffer(2 * maxlength);
        for (int i = 0; i < maxlength; ++i) {
            int ch = (i >= len || password == null) ? '\0' : password.charAt(i);
            ch = (s1++ + ch ^ s2++);
            final int ch2 = ch / 26 + 73;
            final int ch3 = ch % 26 + 65;
            if ((i & 0x1) == 0x1) {
                s2 += (ch2 & 0xF);
            }
            else {
                s2 -= (ch2 & 0xF);
            }
            pw.append((char)ch2);
            pw.append((char)ch3);
        }
        return new String(pw);
    }
    
    public static String decodePassword(final String password) {
        if (password == null) {
            return "";
        }
        int s1 = 97;
        int s2 = 76;
        final int len = password.length() / 2;
        final StringBuffer pw = new StringBuffer();
        for (int i = 0; i < len; ++i) {
            final int ch1 = password.charAt(2 * i);
            final int ch2 = password.charAt(2 * i + 1);
            int ch3 = (ch1 - 73) * 26 + (ch2 - 65);
            ch3 = (ch3 ^ s2++) - s1++;
            if (ch3 == 0) {
                break;
            }
            pw.append((char)ch3);
            if ((i & 0x1) == 0x1) {
                s2 += (ch1 & 0xF);
            }
            else {
                s2 -= (ch1 & 0xF);
            }
        }
        return new String(pw);
    }
    
    public static int[] stringToColRow(final String str) {
        final int[] pos = { 0, 0 };
        for (int i = 0; i < str.length(); ++i) {
            final char ch = str.charAt(i);
            if (ch < 'A' || ch > 'Z') {
                try {
                    pos[1] = Integer.parseInt(str.substring(i));
                }
                catch (final NumberFormatException ex) {
                    pos[1] = 0;
                }
                break;
            }
            pos[0] = pos[0] * 26 + (byte)ch - 65 + 1;
        }
        if (pos[0] > 0 && pos[1] > 0) {
            return pos;
        }
        return null;
    }
    
    public static String colRowToString(int x, final int y) {
        final char[] chrArray = new char[64];
        int len = 0;
        while (x > 0) {
            char ch = (char)(x % 26 + 65 - 1);
            x /= 26;
            if (ch < 'A') {
                ch = 'Z';
                --x;
            }
            chrArray[len] = ch;
            ++len;
        }
        String colStr = "";
        for (int i = 0; i < len; ++i) {
            colStr = String.valueOf(colStr) + chrArray[len - i - 1];
        }
        return String.valueOf(colStr) + Integer.toString(y);
    }
    
    public static String makeHexString(int iv) {
        final char[] buf = new char[8];
        int charPos = 7;
        while (iv != 0) {
            buf[charPos--] = JqLib.HEX_CHARS[iv & 0xF];
            iv >>>= 4;
        }
        while (charPos >= 0) {
            buf[charPos--] = JqLib.HEX_CHARS[0];
        }
        return new String(buf);
    }
    
    public static int parseHexValue(final String hex) {
        return (int)Long.parseLong(hex, 16);
    }
    
    public static String bytesToHexString(final byte[] data) {
        if (data == null) {
            return null;
        }
        final StringBuffer result = new StringBuffer(data.length * 2);
        for (int i = 0; i < data.length; ++i) {
            int b = data[i];
            if (b < 0) {
                b += 256;
            }
            if (b < 16) {
                result.append('0');
            }
            result.append(Integer.toHexString(b));
        }
        return result.toString().toUpperCase();
    }
    
    public static byte[] hexStringToBytes(final String hexString) {
        if (hexString == null) {
            return null;
        }
        final byte[] result = new byte[hexString.length() / 2];
        for (int i = 0; i < result.length; ++i) {
            result[i] = Integer.decode("0x" + hexString.substring(2 * i, 2 * i + 2)).byteValue();
        }
        return result;
    }
    
    public static void listRevert(final List list) {
        final int high = list.size() - 1;
        for (int i = 0; i < list.size() / 2; ++i) {
            final Object o = list.get(i);
            list.set(i, list.get(high - i));
            list.set(high - i, o);
        }
    }
    
    public static void listMove(final List list, final int index1, final int index2) {
        if (index1 > index2) {
            final Object t = list.get(index1);
            for (int i = index1; i > index2; --i) {
                list.set(i, list.get(i - 1));
            }
            list.set(index2, t);
        }
        else if (index1 < index2) {
            final Object t = list.get(index1);
            for (int i = index1; i < index2; ++i) {
                list.set(i, list.get(i + 1));
            }
            list.set(index2, t);
        }
    }
    
    public static int listFind(final List items, final Comparable item) {
        if (item != null) {
            int sb = 0;
            int se = items.size() - 1;
            int flag = -1;
            while (sb <= se) {
                final int sp = sb + se >> 1;
                flag = item.compareTo(items.get(sp));
                if (flag > 0) {
                    sb = sp + 1;
                }
                else {
                    if (flag >= 0) {
                        sb = sp;
                        break;
                    }
                    se = sp - 1;
                }
            }
            return (flag == 0) ? sb : (-sb - 1);
        }
        if (items.size() > 0 && items.get(0) == null) {
            return 0;
        }
        return -1;
    }
    
    public static int listSortAdd(final List items, final Object item) {
        int index = listFind(items, (Comparable)item);
        if (index < 0) {
            index = -index - 1;
            items.add(index, item);
        }
        return index;
    }
    
    public static void listSort(final List items) {
        if (items.size() > 1) {
            listQuickSort(items, 0, items.size() - 1);
        }
    }
    
    protected static void listQuickSort(final List items, int left, final int right) {
        int i;
        do {
            i = left;
            int j = right;
            final Comparable item = items.get(left + right >> 1);
            while (true) {
                if (item.compareTo(items.get(i)) <= 0) {
                    while (item.compareTo(items.get(j)) < 0) {
                        --j;
                    }
                    if (i <= j) {
                        final Object o = items.get(i);
                        items.set(i, items.get(j));
                        items.set(j, o);
                        ++i;
                        --j;
                    }
                    if (i > j) {
                        break;
                    }
                    continue;
                }
                else {
                    ++i;
                }
            }
            if (left < j) {
                listQuickSort(items, left, j);
            }
        } while ((left = i) < right);
    }
    
    public static byte[] stringToBytes(final String s) {
        return (byte[])((s == null) ? null : s.getBytes());
    }
    
    public static String bytesToString(final byte[] b) {
        return (b == null) ? null : new String(b);
    }
    
    public static boolean isLeapYear(final int AYear) {
        return AYear % 4 == 0 && (AYear % 100 != 0 || AYear % 400 == 0);
    }
    
    public static int daysPerMonth(final int AYear, final int AMonth) {
        if (AMonth == 2 && isLeapYear(AYear)) {
            return JqLib.DAYS_IN_MONTH[AMonth] + 1;
        }
        return JqLib.DAYS_IN_MONTH[AMonth];
    }
    
    public static Calendar dayTermToDate(final int AYear, int Term) {
        final Calendar result = new GregorianCalendar();
        result.setTime(new Date(0L));
        result.set(AYear, 1, 1);
        if (Term <= 0) {
            Term = 1;
        }
        for (int I = 1; I <= 12; ++I) {
            if (Term <= daysPerMonth(AYear, I)) {
                result.set(AYear, I, Term);
                break;
            }
            Term -= daysPerMonth(AYear, I);
        }
        return result;
    }
    
    public static String getFileName(final String name) {
        String returnStr = StringUtils.replace(name, "/", "");
        returnStr = StringUtils.replace(returnStr, "\\", "");
        returnStr = StringUtils.replace(returnStr, ":", "");
        returnStr = StringUtils.replace(returnStr, "\"", "");
        returnStr = StringUtils.replace(returnStr, ">", "");
        returnStr = StringUtils.replace(returnStr, "<", "");
        returnStr = StringUtils.replace(returnStr, "|", "");
        returnStr = StringUtils.replace(returnStr, "?", "");
        returnStr = StringUtils.replace(returnStr, "*", "");
        return returnStr;
    }
    
    public static String getChinaNumberTitle(final int value) {
        switch (value) {
            case 0: {
                return "零";
            }
            case 1: {
                return "一";
            }
            case 2: {
                return "二";
            }
            case 3: {
                return "三";
            }
            case 4: {
                return "四";
            }
            case 5: {
                return "五";
            }
            case 6: {
                return "六";
            }
            case 7: {
                return "七";
            }
            case 8: {
                return "八";
            }
            case 9: {
                return "九";
            }
            case 10: {
                return "十";
            }
            default: {
                return "一";
            }
        }
    }
    
    public static void main(final String[] args) {
        String s = "孙建卫";
        final byte[] b = stringToBytes(s);
        System.out.println(b);
        s = bytesToString(b);
        System.out.println(s);
        System.out.println();
    }
}
